<?php

namespace TVn7\NotificationBundle\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;
use TVn7\NotificationBundle\Entity\Notification;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Profiler\Profiler;

use TVn7\NotificationBundle\Services\Notifier;

class NotificationController extends Controller
{

    /**
     *
     * @return ArrayCollection
     */
    public function getHeaderAction()
    {
        $em = $this->getDoctrine()->getManager();

        $notifications = $em->getRepository("TVn7NotificationBundle:Notification")->getLast($this->getUser(), 50);

        return $this->render('TVn7NotificationBundle:Notification:notificationHeader.html.twig', [
            'notifications' => $notifications,
            'notificationsUnRead' => $notifications->filter(function($n) {
                return $n->getReadAt() === null;
            }),
            'last_notif' => $notifications->first()
        ]);
    }

    public function getAction(Request $request, $id, Notification $lastNotification = null)
    {

        if(!$request->isXmlHttpRequest()) {
            throw $this->createNotFoundException();
        }
        if($this->has('profiler')) {
            $this->get('profiler')->disable();
        }

        $data = [];

        $em = $this->getDoctrine()->getManager();

        $data["success"] = true;

        if($lastNotification instanceof Notification) {
            if($this->getUser() !== $lastNotification->getOwner()) {
                throw $this->createAccessDeniedException();
            }
            $notifs = $em->getRepository("TVn7NotificationBundle:Notification")->getLastSince($this->getUser(), $lastNotification);
        } elseif($id == 0) {
            $notifs = $em->getRepository("TVn7NotificationBundle:Notification")->getLast($this->getUser());
        } else {
            throw $this->createNotFoundException();
        }

        $unreaditems = $em->getRepository("TVn7NotificationBundle:Notification")
                           ->findBy(["owner" => $this->getUser(), "readAt" => null]);

        $unread = count($unreaditems);

        $notifications = [];

        foreach($notifs as $notif) {
            $notifications[] = $notif->toArray();
        }

        $uritems = [];
        foreach($unreaditems as $item) {
            $uritems[] = $item->getId();
        }

        $data["notifications"] = $notifications;
        $data["count"] = count($notifications);
        $data["unread"] = $unread;
        $data["unreaditems"] = $uritems;
        $data["lastnotif"] = end($notifications)["id"];

        $response = new JsonResponse($data, Response::HTTP_OK);
        return $response;

    }

    public function readAction(Request $request, Notification $notification)
    {

        if(!$request->isXmlHttpRequest()) {
            throw $this->createNotFoundException();
        }

        if($this->getUser() !== $notification->getOwner()) {
            throw $this->createAccessDeniedException();
        }
        
        $notification->setReadAt(new \DateTime());
        $this->getDoctrine()->getManager()->flush();

        $data["success"] = true;

        $response = new JsonResponse($data, Response::HTTP_OK);
        return $response;

    }

    public function readAllAction(Request $request)
    {

        if(!$request->isXmlHttpRequest()) {
            throw $this->createNotFoundException();
        }

        $now = new \DateTime();

        $em = $this->getDoctrine()->getManager();

        $notifs = $em->getRepository("TVn7NotificationBundle:Notification")
                     ->findBy(["owner" => $this->getUser(), "readAt" => null]);

        array_map(function($notif) use ($now) {
            $notif->setReadAt($now);
        }, $notifs);

        $em->flush();

        $data["success"] = true;

        $response = new JsonResponse($data, Response::HTTP_OK);
        return $response;

    }
}
